#!/usr/bin/env python
# -*- coding: utf-8 -*-
import time
import random

class SubsumptionBehavior:
    def __init__(self):
        self.flag = 0

    def update(self):
        #implemented at subclass
        pass

    def execute(self):
        #implemented at subclass
        pass

    def activate(self):
        self.flag = 1

    def clear(self):
        self.flag = 0

    def is_activated(self):
        return self.flag

class SimpleSubsumptionBrain():
    def __init__(self):
        self.behaviors = []

    def add(self, behavior):
        self.behaviors.append( behavior )

    def step(self):
        b = self.updateAll()
        print "%s is in control" % self.behaviors[b].__class__.__name__
        self.behaviors[b].execute()

    def updateAll(self):
        # for all except lowest:
        for b in range(len(self.behaviors) - 1, 0, -1):
            self.behaviors[b].clear()
            self.behaviors[b].update()
            # if it fired, return number:
            if self.behaviors[b].is_activated():
                return b
        # if none fired, return lowest:
        self.behaviors[0].update()
        return 0

class Sample1(SubsumptionBehavior):
    def update(self):
        self.activate()
        
    def execute(self):
        print 'sample1'

class Sample2(SubsumptionBehavior):
    def __init__(self):
        self.cnt = 0
        SubsumptionBehavior.__init__(self)

    def update(self):
        if self.cnt > 50:
            self.activate()
        else:
            self.cnt += 1

    def execute(self):
        print 'sample2'    

def main():
    ssa = SimpleSubsumptionBrain()
    # add behaviors, lowest priorities first:
    ssa.add( Sample1() )
    ssa.add( Sample2() )
    
    cnt = 0
    while cnt < 100:
        cnt += 1
        ssa.step()
        time.sleep(0.1)

if __name__ == '__main__':
    main()
