#!/usr/bin/env python
# -*- coding: utf-8 -*-
import time
import cv, cv2
import numpy as np
import visionlib
import sensorlib
import motorlib
import simple_subsumption as ss

class SensorBehavior(ss.SubsumptionBehavior):
    def __init__(self):
        ss.SubsumptionBehavior.__init__(self)
        self.collision_thres = 700
        self.speed = 80
        sensorlib.initialize_ADC()    
        
    def update(self):
        vals = sensorlib.read_sensors()
        if vals != None:
            self.rval = vals[0]
            self.lval = vals[1]
            if self.rval > self.collision_thres or self.lval > self.collision_thres:    
                self.activate()

    def execute(self):
        if self.rval > self.collision_thres and self.lval > self.collision_thres:
            print 'stop'
            motorlib.stop_motor()
        elif self.rval > self.collision_thres:
            print 'left'
            motorlib.turn_left(self.speed)
        elif self.lval > self.collision_thres:
            print 'right'
            motorlib.turn_right(self.speed)

class VisionBehavior(ss.SubsumptionBehavior):
    def __init__(self):
        ss.SubsumptionBehavior.__init__(self)
        self.speed = 80
        self.turn_speed = 30
        self.min_area = 500
        self.deadband_x = 75
        self.deadband_area = 2000
        self.target_x = 160
        self.target_area = 5000
        self.capture = cv2.VideoCapture(0)
        self.capture.set(cv.CV_CAP_PROP_FRAME_WIDTH, 320)
        self.capture.set(cv.CV_CAP_PROP_FRAME_HEIGHT, 240)
        time.sleep(0.1)

    def update(self):
        ret, self.img = self.capture.read()
        if ret:
            self.activate()

    def execute(self):
        gx, gy, img_result = visionlib.extract_redregion(self.img, 100, 1.5)
        area = cv2.countNonZero(img_result)        
            
        if gx != -1 and gy != -1 and area > self.min_area:
            diff_x = gx - self.target_x
            diff_area = area - self.target_area

            if abs(diff_x) < self.deadband_x and abs(diff_area) < self.deadband_area:
                print 'stop'
                motorlib.stop_motor()
            elif abs(diff_x) > self.deadband_x:
                if diff_x > 0:
                    print 'right'
                    motorlib.turn_right(self.turn_speed)
                elif diff_x < 0:
                    print 'left'
                    motorlib.turn_left(self.turn_speed)
            else:
                if diff_area > 0:
                    print 'backward'
                    motorlib.go_backward(self.speed)
                elif diff_area < 0:
                    print 'forward'
                    motorlib.go_forward(self.speed)
        else:
            motorlib.stop_motor()        

def main():
    ssa = ss.SimpleSubsumptionBrain()
    ssa.add( VisionBehavior() )
    ssa.add( SensorBehavior() )    
    motorlib.initialize_motor()
    time.sleep(0.1)
    while True:
        try:
            ssa.step()
            time.sleep(0.1)
        except KeyboardInterrupt:
            break
    motorlib.finalize_motor()

if __name__ == '__main__':
    main()

